;;; DO NOT MODIFY THIS FILE
(if (featurep 'mule-ucs-autoloads) (error "Already loaded"))

;;;### (autoloads nil "_pkg" "lisp/_pkg.el")

(package-provide 'mule-ucs :version 1.17 :author-version "0.84" :type 'regular)

;;;***

;;;### (autoloads (un-define-change-charset-order) "un-define" "lisp/un-define.el")

(autoload 'un-define-change-charset-order "un-define" "\
Change UCS to Mule charset conversion priority.

ORDER-LIST is a list of charsets.  When translating a UCS character to a
Mule character, the first charset in the list which contains the character
is used.  ORDER-LIST may be abbreviated to the charsets of interest.  In
this case, charsets in ORDER-LIST are given highest priority in that order,
followed by any charsets not mentioned, with the same relative order as in
the current priority list.

Use `list-character-sets' to get a list of character sets." nil nil)

;;;***

(provide 'mule-ucs-autoloads)
